@@ -1,10 +1,10 @@
-use ruff_diagnostics::{Diagnostic, Violation};
+use ruff_diagnostics::{Diagnostic, Fix, FixAvailability, Violation};
 use ruff_macros::{derive_message_formats, violation};
-
-use ruff_python_ast::{self as ast};
+use ruff_python_ast as ast;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
+use crate::fix::edits::delete_stmt;
 use crate::registry::AsRule;
 
 /// ## What it does
@@ -28,14 +28,24 @@ use crate::registry::AsRule;
 /// def add_numbers(a, b):
 ///     return a + b
 /// ```
+///
+/// ## Fix safety
+/// This rule's fix is marked as unsafe, as it may remove `print` statements
+/// that are used beyond debugging purposes.
 #[violation]
 pub struct Print;
 
 impl Violation for Print {
+    const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
+
     #[derive_message_formats]
     fn message(&self) -> String {
         format!("`print` found")
     }
+
+    fn fix_title(&self) -> Option<String> {
+        Some("Remove `print`".to_string())
+    }
 }
 
 /// ## What it does
@@ -65,19 +75,29 @@ impl Violation for Print {
 ///     dict_c = {**dict_a, **dict_b}
 ///     return dict_c
 /// ```
+///
+/// ## Fix safety
+/// This rule's fix is marked as unsafe, as it may remove `pprint` statements
+/// that are used beyond debugging purposes.
 #[violation]
 pub struct PPrint;
 
 impl Violation for PPrint {
+    const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
+
     #[derive_message_formats]
     fn message(&self) -> String {
         format!("`pprint` found")
     }
+
+    fn fix_title(&self) -> Option<String> {
+        Some("Remove `pprint`".to_string())
+    }
 }
 
 /// T201, T203
 pub(crate) fn print_call(checker: &mut Checker, call: &ast::ExprCall) {
-    let diagnostic = {
+    let mut diagnostic = {
         let call_path = checker.semantic().resolve_call_path(&call.func);
         if call_path
             .as_ref()
@@ -113,5 +133,15 @@ pub(crate) fn print_call(checker: &mut Checker, call: &ast::ExprCall) {
         return;
     }
 
+    // Remove the `print`, if it's a standalone statement.
+    if checker.semantic().current_expression_parent().is_none() {
+        let statement = checker.semantic().current_statement();
+        let parent = checker.semantic().current_statement_parent();
+        let edit = delete_stmt(statement, parent, checker.locator(), checker.indexer());
+        diagnostic.set_fix(Fix::unsafe_edit(edit).isolate(Checker::isolation(
+            checker.semantic().current_statement_parent_id(),
+        )));
+    }
+
     checker.diagnostics.push(diagnostic);
 }
