@@ -1,13 +1,12 @@
 use anyhow::Result;
-use rustc_hash::FxHashSet;
 
 use ruff_diagnostics::Edit;
 use ruff_python_ast::call_path::from_qualified_name;
-use ruff_python_ast::helpers::{map_callable, map_subscript};
-use ruff_python_ast::{self as ast, Expr};
-use ruff_python_codegen::Stylist;
+use ruff_python_ast::helpers::map_callable;
+use ruff_python_ast::{self as ast, Decorator, Expr};
+use ruff_python_codegen::{Generator, Stylist};
 use ruff_python_semantic::{
-    Binding, BindingId, BindingKind, NodeId, ResolvedReference, SemanticModel,
+    analyze, Binding, BindingKind, NodeId, ResolvedReference, SemanticModel,
 };
 use ruff_source_file::Locator;
 use ruff_text_size::Ranged;
@@ -44,6 +43,19 @@ pub(crate) fn is_valid_runtime_import(
     }
 }
 
+/// Returns `true` if a function's parameters should be treated as runtime-required.
+pub(crate) fn runtime_required_function(
+    function_def: &ast::StmtFunctionDef,
+    decorators: &[String],
+    semantic: &SemanticModel,
+) -> bool {
+    if runtime_required_decorators(&function_def.decorator_list, decorators, semantic) {
+        return true;
+    }
+    false
+}
+
+/// Returns `true` if a class's assignments should be treated as runtime-required.
 pub(crate) fn runtime_required_class(
     class_def: &ast::StmtClassDef,
     base_classes: &[String],
@@ -53,67 +65,27 @@ pub(crate) fn runtime_required_class(
     if runtime_required_base_class(class_def, base_classes, semantic) {
         return true;
     }
-    if runtime_required_decorators(class_def, decorators, semantic) {
+    if runtime_required_decorators(&class_def.decorator_list, decorators, semantic) {
         return true;
     }
     false
 }
 
+/// Return `true` if a class is a subclass of a runtime-required base class.
 fn runtime_required_base_class(
     class_def: &ast::StmtClassDef,
     base_classes: &[String],
     semantic: &SemanticModel,
 ) -> bool {
-    fn inner(
-        class_def: &ast::StmtClassDef,
-        base_classes: &[String],
-        semantic: &SemanticModel,
-        seen: &mut FxHashSet<BindingId>,
-    ) -> bool {
-        class_def.bases().iter().any(|expr| {
-            // If the base class is itself runtime-required, then this is too.
-            // Ex) `class Foo(BaseModel): ...`
-            if semantic
-                .resolve_call_path(map_subscript(expr))
-                .is_some_and(|call_path| {
-                    base_classes
-                        .iter()
-                        .any(|base_class| from_qualified_name(base_class) == call_path)
-                })
-            {
-                return true;
-            }
-
-            // If the base class extends a runtime-required class, then this does too.
-            // Ex) `class Bar(BaseModel): ...; class Foo(Bar): ...`
-            if let Some(id) = semantic.lookup_attribute(map_subscript(expr)) {
-                if seen.insert(id) {
-                    let binding = semantic.binding(id);
-                    if let Some(base_class) = binding
-                        .kind
-                        .as_class_definition()
-                        .map(|id| &semantic.scopes[*id])
-                        .and_then(|scope| scope.kind.as_class())
-                    {
-                        if inner(base_class, base_classes, semantic, seen) {
-                            return true;
-                        }
-                    }
-                }
-            }
-            false
-        })
-    }
-
-    if base_classes.is_empty() {
-        return false;
-    }
-
-    inner(class_def, base_classes, semantic, &mut FxHashSet::default())
+    analyze::class::any_call_path(class_def, semantic, &|call_path| {
+        base_classes
+            .iter()
+            .any(|base_class| from_qualified_name(base_class) == call_path)
+    })
 }
 
 fn runtime_required_decorators(
-    class_def: &ast::StmtClassDef,
+    decorator_list: &[Decorator],
     decorators: &[String],
     semantic: &SemanticModel,
 ) -> bool {
@@ -121,7 +93,7 @@ fn runtime_required_decorators(
         return false;
     }
 
-    class_def.decorator_list.iter().any(|decorator| {
+    decorator_list.iter().any(|decorator| {
         semantic
             .resolve_call_path(map_callable(&decorator.expression))
             .is_some_and(|call_path| {
@@ -215,6 +187,7 @@ pub(crate) fn quote_annotation(
     semantic: &SemanticModel,
     locator: &Locator,
     stylist: &Stylist,
+    generator: Generator,
 ) -> Result<Edit> {
     let expr = semantic.expression(node_id).expect("Expression not found");
     if let Some(parent_id) = semantic.parent_expression_id(node_id) {
@@ -224,7 +197,7 @@ pub(crate) fn quote_annotation(
                     // If we're quoting the value of a subscript, we need to quote the entire
                     // expression. For example, when quoting `DataFrame` in `DataFrame[int]`, we
                     // should generate `"DataFrame[int]"`.
-                    return quote_annotation(parent_id, semantic, locator, stylist);
+                    return quote_annotation(parent_id, semantic, locator, stylist, generator);
                 }
             }
             Some(Expr::Attribute(parent)) => {
@@ -232,7 +205,7 @@ pub(crate) fn quote_annotation(
                     // If we're quoting the value of an attribute, we need to quote the entire
                     // expression. For example, when quoting `DataFrame` in `pd.DataFrame`, we
                     // should generate `"pd.DataFrame"`.
-                    return quote_annotation(parent_id, semantic, locator, stylist);
+                    return quote_annotation(parent_id, semantic, locator, stylist, generator);
                 }
             }
             Some(Expr::Call(parent)) => {
@@ -240,7 +213,7 @@ pub(crate) fn quote_annotation(
                     // If we're quoting the function of a call, we need to quote the entire
                     // expression. For example, when quoting `DataFrame` in `DataFrame()`, we
                     // should generate `"DataFrame()"`.
-                    return quote_annotation(parent_id, semantic, locator, stylist);
+                    return quote_annotation(parent_id, semantic, locator, stylist, generator);
                 }
             }
             Some(Expr::BinOp(parent)) => {
@@ -248,27 +221,44 @@ pub(crate) fn quote_annotation(
                     // If we're quoting the left or right side of a binary operation, we need to
                     // quote the entire expression. For example, when quoting `DataFrame` in
                     // `DataFrame | Series`, we should generate `"DataFrame | Series"`.
-                    return quote_annotation(parent_id, semantic, locator, stylist);
+                    return quote_annotation(parent_id, semantic, locator, stylist, generator);
                 }
             }
             _ => {}
         }
     }
 
-    let annotation = locator.slice(expr);
-
     // If the annotation already contains a quote, avoid attempting to re-quote it. For example:
     // ```python
     // from typing import Literal
     //
     // Set[Literal["Foo"]]
     // ```
-    if annotation.contains('\'') || annotation.contains('"') {
+    let text = locator.slice(expr);
+    if text.contains('\'') || text.contains('"') {
         return Err(anyhow::anyhow!("Annotation already contains a quote"));
     }
 
-    // If we're quoting a name, we need to quote the entire expression.
+    // Quote the entire expression.
     let quote = stylist.quote();
-    let annotation = format!("{quote}{annotation}{quote}");
-    Ok(Edit::range_replacement(annotation, expr.range()))
+    let annotation = generator.expr(expr);
+
+    Ok(Edit::range_replacement(
+        format!("{quote}{annotation}{quote}"),
+        expr.range(),
+    ))
+}
+
+/// Filter out any [`Edit`]s that are completely contained by any other [`Edit`].
+pub(crate) fn filter_contained(edits: Vec<Edit>) -> Vec<Edit> {
+    let mut filtered: Vec<Edit> = Vec::with_capacity(edits.len());
+    for edit in edits {
+        if filtered
+            .iter()
+            .all(|filtered_edit| !filtered_edit.range().contains_range(edit.range()))
+        {
+            filtered.push(edit);
+        }
+    }
+    filtered
 }
