@@ -12,7 +12,7 @@ use crate::checkers::ast::Checker;
 use crate::codes::Rule;
 use crate::fix;
 use crate::importer::ImportedMembers;
-use crate::rules::flake8_type_checking::helpers::quote_annotation;
+use crate::rules::flake8_type_checking::helpers::{filter_contained, quote_annotation};
 use crate::rules::flake8_type_checking::imports::ImportBinding;
 
 /// ## What it does
@@ -262,32 +262,33 @@ pub(crate) fn runtime_import_in_type_checking_block(
 
 /// Generate a [`Fix`] to quote runtime usages for imports in a type-checking block.
 fn quote_imports(checker: &Checker, node_id: NodeId, imports: &[ImportBinding]) -> Result<Fix> {
-    let mut quote_reference_edits = imports
-        .iter()
-        .flat_map(|ImportBinding { binding, .. }| {
-            binding.references.iter().filter_map(|reference_id| {
-                let reference = checker.semantic().reference(*reference_id);
-                if reference.context().is_runtime() {
-                    Some(quote_annotation(
-                        reference.expression_id()?,
-                        checker.semantic(),
-                        checker.locator(),
-                        checker.stylist(),
-                    ))
-                } else {
-                    None
-                }
+    let quote_reference_edits = filter_contained(
+        imports
+            .iter()
+            .flat_map(|ImportBinding { binding, .. }| {
+                binding.references.iter().filter_map(|reference_id| {
+                    let reference = checker.semantic().reference(*reference_id);
+                    if reference.context().is_runtime() {
+                        Some(quote_annotation(
+                            reference.expression_id()?,
+                            checker.semantic(),
+                            checker.locator(),
+                            checker.stylist(),
+                            checker.generator(),
+                        ))
+                    } else {
+                        None
+                    }
+                })
             })
-        })
-        .collect::<Result<Vec<_>>>()?;
-    let quote_reference_edit = quote_reference_edits
-        .pop()
-        .expect("Expected at least one reference");
-    Ok(
-        Fix::unsafe_edits(quote_reference_edit, quote_reference_edits).isolate(Checker::isolation(
-            checker.semantic().parent_statement_id(node_id),
-        )),
-    )
+            .collect::<Result<Vec<_>>>()?,
+    );
+
+    let mut rest = quote_reference_edits.into_iter();
+    let head = rest.next().expect("Expected at least one reference");
+    Ok(Fix::unsafe_edits(head, rest).isolate(Checker::isolation(
+        checker.semantic().parent_statement_id(node_id),
+    )))
 }
 
 /// Generate a [`Fix`] to remove runtime imports from a type-checking block.
