@@ -8,7 +8,6 @@ use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
 use crate::docstrings::Docstring;
-use crate::registry::AsRule;
 
 /// ## What it does
 /// Checks for multi-line docstrings whose closing quotes are not on their
@@ -81,27 +80,25 @@ pub(crate) fn newline_after_last_paragraph(checker: &mut Checker, docstring: &Do
                 if last_line != "\"\"\"" && last_line != "'''" {
                     let mut diagnostic =
                         Diagnostic::new(NewLineAfterLastParagraph, docstring.range());
-                    if checker.patch(diagnostic.kind.rule()) {
-                        // Insert a newline just before the end-quote(s).
-                        let num_trailing_quotes = "'''".text_len();
-                        let num_trailing_spaces: TextSize = last_line
-                            .chars()
-                            .rev()
-                            .skip(usize::from(num_trailing_quotes))
-                            .take_while(|c| c.is_whitespace())
-                            .map(TextLen::text_len)
-                            .sum();
-                        let content = format!(
-                            "{}{}",
-                            checker.stylist().line_ending().as_str(),
-                            clean_space(docstring.indentation)
-                        );
-                        diagnostic.set_fix(Fix::automatic(Edit::replacement(
-                            content,
-                            docstring.expr.end() - num_trailing_quotes - num_trailing_spaces,
-                            docstring.expr.end() - num_trailing_quotes,
-                        )));
-                    }
+                    // Insert a newline just before the end-quote(s).
+                    let num_trailing_quotes = "'''".text_len();
+                    let num_trailing_spaces: TextSize = last_line
+                        .chars()
+                        .rev()
+                        .skip(usize::from(num_trailing_quotes))
+                        .take_while(|c| c.is_whitespace())
+                        .map(TextLen::text_len)
+                        .sum();
+                    let content = format!(
+                        "{}{}",
+                        checker.stylist().line_ending().as_str(),
+                        clean_space(docstring.indentation)
+                    );
+                    diagnostic.set_fix(Fix::safe_edit(Edit::replacement(
+                        content,
+                        docstring.end() - num_trailing_quotes - num_trailing_spaces,
+                        docstring.end() - num_trailing_quotes,
+                    )));
                     checker.diagnostics.push(diagnostic);
                 }
             }
