@@ -14,11 +14,13 @@ pub(crate) use comparison_of_constant::*;
 pub(crate) use comparison_with_itself::*;
 pub(crate) use continue_in_finally::*;
 pub(crate) use duplicate_bases::*;
+pub(crate) use empty_comment::*;
 pub(crate) use eq_without_hash::*;
 pub(crate) use global_at_module_level::*;
 pub(crate) use global_statement::*;
 pub(crate) use global_variable_not_assigned::*;
 pub(crate) use import_outside_top_level::*;
+pub(crate) use import_private_name::*;
 pub(crate) use import_self::*;
 pub(crate) use invalid_all_format::*;
 pub(crate) use invalid_all_object::*;
@@ -51,10 +53,12 @@ pub(crate) use self_assigning_variable::*;
 pub(crate) use single_string_slots::*;
 pub(crate) use subprocess_popen_preexec_fn::*;
 pub(crate) use subprocess_run_without_check::*;
+pub(crate) use super_without_brackets::*;
 pub(crate) use sys_exit_alias::*;
 pub(crate) use too_many_arguments::*;
 pub(crate) use too_many_boolean_expressions::*;
 pub(crate) use too_many_branches::*;
+pub(crate) use too_many_locals::*;
 pub(crate) use too_many_positional::*;
 pub(crate) use too_many_public_methods::*;
 pub(crate) use too_many_return_statements::*;
@@ -65,6 +69,7 @@ pub(crate) use type_param_name_mismatch::*;
 pub(crate) use unexpected_special_method_signature::*;
 pub(crate) use unnecessary_dict_index_lookup::*;
 pub(crate) use unnecessary_direct_lambda_call::*;
+pub(crate) use unnecessary_dunder_call::*;
 pub(crate) use unnecessary_lambda::*;
 pub(crate) use unnecessary_list_index_lookup::*;
 pub(crate) use unspecified_encoding::*;
@@ -91,11 +96,13 @@ mod comparison_of_constant;
 mod comparison_with_itself;
 mod continue_in_finally;
 mod duplicate_bases;
+mod empty_comment;
 mod eq_without_hash;
 mod global_at_module_level;
 mod global_statement;
 mod global_variable_not_assigned;
 mod import_outside_top_level;
+mod import_private_name;
 mod import_self;
 mod invalid_all_format;
 mod invalid_all_object;
@@ -128,10 +135,12 @@ mod self_assigning_variable;
 mod single_string_slots;
 mod subprocess_popen_preexec_fn;
 mod subprocess_run_without_check;
+mod super_without_brackets;
 mod sys_exit_alias;
 mod too_many_arguments;
 mod too_many_boolean_expressions;
 mod too_many_branches;
+mod too_many_locals;
 mod too_many_positional;
 mod too_many_public_methods;
 mod too_many_return_statements;
@@ -142,6 +151,7 @@ mod type_param_name_mismatch;
 mod unexpected_special_method_signature;
 mod unnecessary_dict_index_lookup;
 mod unnecessary_direct_lambda_call;
+mod unnecessary_dunder_call;
 mod unnecessary_lambda;
 mod unnecessary_list_index_lookup;
 mod unspecified_encoding;
