@@ -1,8 +1,8 @@
-use ruff_python_ast::{Parameters, Stmt};
-
 use ruff_diagnostics::{Diagnostic, Violation};
 use ruff_macros::{derive_message_formats, violation};
+use ruff_python_ast as ast;
 use ruff_python_ast::identifier::Identifier;
+use ruff_python_semantic::analyze::visibility;
 
 use crate::checkers::ast::Checker;
 
@@ -58,12 +58,13 @@ impl Violation for TooManyArguments {
 }
 
 /// PLR0913
-pub(crate) fn too_many_arguments(checker: &mut Checker, parameters: &Parameters, stmt: &Stmt) {
-    let num_arguments = parameters
+pub(crate) fn too_many_arguments(checker: &mut Checker, function_def: &ast::StmtFunctionDef) {
+    let num_arguments = function_def
+        .parameters
         .args
         .iter()
-        .chain(&parameters.kwonlyargs)
-        .chain(&parameters.posonlyargs)
+        .chain(&function_def.parameters.kwonlyargs)
+        .chain(&function_def.parameters.posonlyargs)
         .filter(|arg| {
             !checker
                 .settings
@@ -71,13 +72,22 @@ pub(crate) fn too_many_arguments(checker: &mut Checker, parameters: &Parameters,
                 .is_match(&arg.parameter.name)
         })
         .count();
+
     if num_arguments > checker.settings.pylint.max_args {
+        // Allow excessive arguments in `@override` or `@overload` methods, since they're required
+        // to adhere to the parent signature.
+        if visibility::is_override(&function_def.decorator_list, checker.semantic())
+            || visibility::is_overload(&function_def.decorator_list, checker.semantic())
+        {
+            return;
+        }
+
         checker.diagnostics.push(Diagnostic::new(
             TooManyArguments {
                 c_args: num_arguments,
                 max_args: checker.settings.pylint.max_args,
             },
-            stmt.identifier(),
+            function_def.identifier(),
         ));
     }
 }
