@@ -9,12 +9,14 @@ pub(crate) use invalid_index_type::*;
 pub(crate) use invalid_pyproject_toml::*;
 pub(crate) use mutable_class_default::*;
 pub(crate) use mutable_dataclass_default::*;
+pub(crate) use never_union::*;
 pub(crate) use pairwise_over_zipped::*;
+pub(crate) use parenthesize_logical_operators::*;
+pub(crate) use quadratic_list_summation::*;
+pub(crate) use sort_dunder_all::*;
 pub(crate) use static_key_dict_comprehension::*;
 pub(crate) use unnecessary_iterable_allocation_for_first_element::*;
 pub(crate) use unnecessary_key_check::*;
-#[cfg(feature = "unreachable-code")]
-pub(crate) use unreachable::*;
 pub(crate) use unused_noqa::*;
 
 mod ambiguous_unicode_character;
@@ -30,12 +32,13 @@ mod invalid_index_type;
 mod invalid_pyproject_toml;
 mod mutable_class_default;
 mod mutable_dataclass_default;
+mod never_union;
 mod pairwise_over_zipped;
+mod parenthesize_logical_operators;
+mod sort_dunder_all;
 mod static_key_dict_comprehension;
 mod unnecessary_iterable_allocation_for_first_element;
 mod unnecessary_key_check;
-#[cfg(feature = "unreachable-code")]
-pub(crate) mod unreachable;
 mod unused_noqa;
 
 #[derive(Clone, Copy)]
@@ -44,6 +47,5 @@ pub(crate) enum Context {
     Docstring,
     Comment,
 }
-pub(crate) use quadratic_list_summation::*;
 
 mod quadratic_list_summation;
