@@ -171,48 +171,43 @@ impl Cell {
 
         // Detect cell magics (which operate on multiple lines).
         lines.any(|line| {
-            line.split_whitespace().next().is_some_and(|first| {
-                if first.len() < 2 {
-                    return false;
-                }
-                let (token, command) = first.split_at(2);
-                // These cell magics are special in that the lines following them are valid
-                // Python code and the variables defined in that scope are available to the
-                // rest of the notebook.
-                //
-                // For example:
-                //
-                // Cell 1:
-                // ```python
-                // x = 1
-                // ```
-                //
-                // Cell 2:
-                // ```python
-                // %%time
-                // y = x
-                // ```
-                //
-                // Cell 3:
-                // ```python
-                // print(y)  # Here, `y` is available.
-                // ```
-                //
-                // This is to avoid false positives when these variables are referenced
-                // elsewhere in the notebook.
-                token == "%%"
-                    && !matches!(
-                        command,
-                        "capture"
-                            | "debug"
-                            | "prun"
-                            | "pypy"
-                            | "python"
-                            | "python3"
-                            | "time"
-                            | "timeit"
-                    )
-            })
+            let Some(first) = line.split_whitespace().next() else {
+                return false;
+            };
+            if first.len() < 2 {
+                return false;
+            }
+            let Some(command) = first.strip_prefix("%%") else {
+                return false;
+            };
+            // These cell magics are special in that the lines following them are valid
+            // Python code and the variables defined in that scope are available to the
+            // rest of the notebook.
+            //
+            // For example:
+            //
+            // Cell 1:
+            // ```python
+            // x = 1
+            // ```
+            //
+            // Cell 2:
+            // ```python
+            // %%time
+            // y = x
+            // ```
+            //
+            // Cell 3:
+            // ```python
+            // print(y)  # Here, `y` is available.
+            // ```
+            //
+            // This is to avoid false positives when these variables are referenced
+            // elsewhere in the notebook.
+            !matches!(
+                command,
+                "capture" | "debug" | "prun" | "pypy" | "python" | "python3" | "time" | "timeit"
+            )
         })
     }
 }
