@@ -126,7 +126,7 @@ where
     FormatParenthesized {
         left,
         comments: &[],
-        indent: true,
+        hug: None,
         content: Argument::new(content),
         right,
     }
@@ -135,7 +135,7 @@ where
 pub(crate) struct FormatParenthesized<'content, 'ast> {
     left: &'static str,
     comments: &'content [SourceComment],
-    indent: bool,
+    hug: Option<HuggingStyle>,
     content: Argument<'content, PyFormatContext<'ast>>,
     right: &'static str,
 }
@@ -158,8 +158,11 @@ impl<'content, 'ast> FormatParenthesized<'content, 'ast> {
     }
 
     /// Whether to indent the content within the parentheses.
-    pub(crate) fn with_indent(self, indent: bool) -> FormatParenthesized<'content, 'ast> {
-        FormatParenthesized { indent, ..self }
+    pub(crate) fn with_hugging(
+        self,
+        hug: Option<HuggingStyle>,
+    ) -> FormatParenthesized<'content, 'ast> {
+        FormatParenthesized { hug, ..self }
     }
 }
 
@@ -167,17 +170,41 @@ impl<'ast> Format<PyFormatContext<'ast>> for FormatParenthesized<'_, 'ast> {
     fn fmt(&self, f: &mut Formatter<PyFormatContext<'ast>>) -> FormatResult<()> {
         let current_level = f.context().node_level();
 
-        let content = format_with(|f| {
-            group(&format_with(|f| {
-                dangling_open_parenthesis_comments(self.comments).fmt(f)?;
-                if self.indent || !self.comments.is_empty() {
-                    soft_block_indent(&Arguments::from(&self.content)).fmt(f)?;
-                } else {
-                    Arguments::from(&self.content).fmt(f)?;
+        let indented = format_with(|f| {
+            let content = Arguments::from(&self.content);
+            if self.comments.is_empty() {
+                match self.hug {
+                    None => group(&soft_block_indent(&content)).fmt(f),
+                    Some(HuggingStyle::Always) => content.fmt(f),
+                    Some(HuggingStyle::IfFirstLineFits) => {
+                        // It's not immediately obvious how the below IR works to only indent the content if the first line exceeds the configured line width.
+                        // The trick is the first group that doesn't wrap `self.content`.
+                        // * The group doesn't wrap `self.content` because we need to assume that `self.content`
+                        //   contains a hard line break and hard-line-breaks always expand the enclosing group.
+                        // * The printer decides that a group fits if its content (in this case a `soft_line_break` that has a width of 0 and is guaranteed to fit)
+                        //   and the content coming after the group in expanded mode (`self.content`) fits on the line.
+                        //   The content coming after fits if the content up to the first soft or hard line break (or the end of the document) fits.
+                        //
+                        // This happens to be right what we want. The first group should add an indent and a soft line break if the content of `self.content`
+                        // up to the first line break exceeds the configured line length, but not otherwise.
+                        let indented = f.group_id("indented_content");
+                        write!(
+                            f,
+                            [
+                                group(&indent(&soft_line_break())).with_group_id(Some(indented)),
+                                indent_if_group_breaks(&content, indented),
+                                if_group_breaks(&soft_line_break()).with_group_id(Some(indented))
+                            ]
+                        )
+                    }
                 }
-                Ok(())
-            }))
-            .fmt(f)
+            } else {
+                group(&format_args![
+                    dangling_open_parenthesis_comments(self.comments),
+                    soft_block_indent(&content),
+                ])
+                .fmt(f)
+            }
         });
 
         let inner = format_with(|f| {
@@ -186,12 +213,12 @@ impl<'ast> Format<PyFormatContext<'ast>> for FormatParenthesized<'_, 'ast> {
                 // This ensures that expanding this parenthesized expression does not expand the optional parentheses group.
                 write!(
                     f,
-                    [fits_expanded(&content)
+                    [fits_expanded(&indented)
                         .with_condition(Some(Condition::if_group_fits_on_line(group_id)))]
                 )
             } else {
                 // It's not necessary to wrap the content if it is not inside of an optional_parentheses group.
-                content.fmt(f)
+                indented.fmt(f)
             }
         });
 
@@ -201,6 +228,20 @@ impl<'ast> Format<PyFormatContext<'ast>> for FormatParenthesized<'_, 'ast> {
     }
 }
 
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+pub(crate) enum HuggingStyle {
+    /// Always hug the content (never indent).
+    Always,
+
+    /// Hug the content if the content up to the first line break fits into the configured line length. Otherwise indent the content.
+    ///
+    /// This is different from [`HuggingStyle::Always`] in that it doesn't indent if the content contains a hard line break, and the content up to that hard line break fits into the configured line length.
+    ///
+    /// This style is used for formatting multiline strings that, by definition, always break. The idea is to
+    /// only hug a multiline string if its content up to the first line breaks exceeds the configured line length.
+    IfFirstLineFits,
+}
+
 /// Wraps an expression in parentheses only if it still does not fit after expanding all expressions that start or end with
 /// a parentheses (`()`, `[]`, `{}`).
 pub(crate) fn optional_parentheses<'content, 'ast, Content>(
@@ -447,7 +488,7 @@ mod tests {
     #[test]
     fn test_has_parentheses() {
         let expression = r#"(b().c("")).d()"#;
-        let expr = parse_expression(expression, "<filename>").unwrap();
+        let expr = parse_expression(expression).unwrap();
         assert!(!is_expression_parenthesized(
             ExpressionRef::from(&expr),
             &CommentRanges::default(),
