@@ -6,8 +6,7 @@ use ruff_formatter::{format, FormatError, Formatted, PrintError, Printed, Source
 use ruff_python_ast::AstNode;
 use ruff_python_ast::Mod;
 use ruff_python_index::tokens_and_ranges;
-use ruff_python_parser::lexer::LexicalError;
-use ruff_python_parser::{parse_ok_tokens, AsMode, ParseError};
+use ruff_python_parser::{parse_tokens, AsMode, ParseError, ParseErrorType};
 use ruff_python_trivia::CommentRanges;
 use ruff_source_file::Locator;
 
@@ -17,7 +16,7 @@ use crate::comments::{
 pub use crate::context::PyFormatContext;
 pub use crate::options::{
     DocstringCode, DocstringCodeLineWidth, MagicTrailingComma, PreviewMode, PyFormatOptions,
-    QuoteStyle,
+    PythonVersion, QuoteStyle,
 };
 pub use crate::shared_traits::{AsFormat, FormattedIter, FormattedIterExt, IntoFormat};
 use crate::verbatim::suppressed_node;
@@ -36,6 +35,7 @@ mod prelude;
 mod preview;
 mod shared_traits;
 pub(crate) mod statement;
+pub(crate) mod string;
 pub(crate) mod type_param;
 mod verbatim;
 
@@ -107,36 +107,26 @@ where
 
 #[derive(Error, Debug)]
 pub enum FormatModuleError {
-    #[error("source contains syntax errors: {0:?}")]
-    LexError(LexicalError),
-    #[error("source contains syntax errors: {0:?}")]
-    ParseError(ParseError),
+    #[error(transparent)]
+    ParseError(#[from] ParseError),
     #[error(transparent)]
     FormatError(#[from] FormatError),
     #[error(transparent)]
     PrintError(#[from] PrintError),
 }
 
-impl From<LexicalError> for FormatModuleError {
-    fn from(value: LexicalError) -> Self {
-        Self::LexError(value)
-    }
-}
-
-impl From<ParseError> for FormatModuleError {
-    fn from(value: ParseError) -> Self {
-        Self::ParseError(value)
-    }
-}
-
 #[tracing::instrument(name = "format", level = Level::TRACE, skip_all)]
 pub fn format_module_source(
     source: &str,
     options: PyFormatOptions,
 ) -> Result<Printed, FormatModuleError> {
     let source_type = options.source_type();
-    let (tokens, comment_ranges) = tokens_and_ranges(source, source_type)?;
-    let module = parse_ok_tokens(tokens, source, source_type.as_mode(), "<filename>")?;
+    let (tokens, comment_ranges) =
+        tokens_and_ranges(source, source_type).map_err(|err| ParseError {
+            offset: err.location,
+            error: ParseErrorType::Lexical(err.error),
+        })?;
+    let module = parse_tokens(tokens, source, source_type.as_mode())?;
     let formatted = format_module_ast(&module, &comment_ranges, source, options)?;
     Ok(formatted.print()?)
 }
@@ -179,8 +169,7 @@ mod tests {
 
     use ruff_python_ast::PySourceType;
     use ruff_python_index::tokens_and_ranges;
-
-    use ruff_python_parser::{parse_ok_tokens, AsMode};
+    use ruff_python_parser::{parse_tokens, AsMode};
 
     use crate::{format_module_ast, format_module_source, PyFormatOptions};
 
@@ -224,7 +213,7 @@ def main() -> None:
 
         // Parse the AST.
         let source_path = "code_inline.py";
-        let module = parse_ok_tokens(tokens, source, source_type.as_mode(), source_path).unwrap();
+        let module = parse_tokens(tokens, source, source_type.as_mode()).unwrap();
         let options = PyFormatOptions::from_extension(Path::new(source_path));
         let formatted = format_module_ast(&module, &comment_ranges, source, options).unwrap();
 
