@@ -2,6 +2,7 @@ use ruff_formatter::printer::{LineEnding, PrinterOptions, SourceMapGeneration};
 use ruff_formatter::{FormatOptions, IndentStyle, IndentWidth, LineWidth};
 use ruff_macros::CacheKey;
 use ruff_python_ast::PySourceType;
+use std::fmt;
 use std::path::Path;
 use std::str::FromStr;
 
@@ -17,6 +18,10 @@ pub struct PyFormatOptions {
     /// Whether we're in a `.py` file or `.pyi` file, which have different rules.
     source_type: PySourceType,
 
+    /// The (minimum) Python version used to run the formatted code. This is used
+    /// to determine the supported Python syntax.
+    target_version: PythonVersion,
+
     /// Specifies the indent style:
     /// * Either a tab
     /// * or a specific amount of spaces
@@ -74,6 +79,7 @@ impl Default for PyFormatOptions {
     fn default() -> Self {
         Self {
             source_type: PySourceType::default(),
+            target_version: PythonVersion::default(),
             indent_style: default_indent_style(),
             line_width: default_line_width(),
             indent_width: default_indent_width(),
@@ -101,31 +107,35 @@ impl PyFormatOptions {
         }
     }
 
-    pub fn magic_trailing_comma(&self) -> MagicTrailingComma {
+    pub const fn target_version(&self) -> PythonVersion {
+        self.target_version
+    }
+
+    pub const fn magic_trailing_comma(&self) -> MagicTrailingComma {
         self.magic_trailing_comma
     }
 
-    pub fn quote_style(&self) -> QuoteStyle {
+    pub const fn quote_style(&self) -> QuoteStyle {
         self.quote_style
     }
 
-    pub fn source_type(&self) -> PySourceType {
+    pub const fn source_type(&self) -> PySourceType {
         self.source_type
     }
 
-    pub fn source_map_generation(&self) -> SourceMapGeneration {
+    pub const fn source_map_generation(&self) -> SourceMapGeneration {
         self.source_map_generation
     }
 
-    pub fn line_ending(&self) -> LineEnding {
+    pub const fn line_ending(&self) -> LineEnding {
         self.line_ending
     }
 
-    pub fn docstring_code(&self) -> DocstringCode {
+    pub const fn docstring_code(&self) -> DocstringCode {
         self.docstring_code
     }
 
-    pub fn docstring_code_line_width(&self) -> DocstringCodeLineWidth {
+    pub const fn docstring_code_line_width(&self) -> DocstringCodeLineWidth {
         self.docstring_code_line_width
     }
 
@@ -133,6 +143,12 @@ impl PyFormatOptions {
         self.preview
     }
 
+    #[must_use]
+    pub fn with_target_version(mut self, target_version: PythonVersion) -> Self {
+        self.target_version = target_version;
+        self
+    }
+
     #[must_use]
     pub fn with_indent_width(mut self, indent_width: IndentWidth) -> Self {
         self.indent_width = indent_width;
@@ -226,6 +242,16 @@ pub enum QuoteStyle {
     Preserve,
 }
 
+impl fmt::Display for QuoteStyle {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Single => write!(f, "single"),
+            Self::Double => write!(f, "double"),
+            Self::Preserve => write!(f, "preserve"),
+        }
+    }
+}
+
 impl FromStr for QuoteStyle {
     type Err = &'static str;
 
@@ -262,6 +288,15 @@ impl MagicTrailingComma {
     }
 }
 
+impl fmt::Display for MagicTrailingComma {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Respect => write!(f, "respect"),
+            Self::Ignore => write!(f, "ignore"),
+        }
+    }
+}
+
 impl FromStr for MagicTrailingComma {
     type Err = &'static str;
 
@@ -291,6 +326,15 @@ impl PreviewMode {
     }
 }
 
+impl fmt::Display for PreviewMode {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Disabled => write!(f, "disabled"),
+            Self::Enabled => write!(f, "enabled"),
+        }
+    }
+}
+
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Default, CacheKey)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[cfg_attr(feature = "serde", serde(rename_all = "lowercase"))]
@@ -308,6 +352,15 @@ impl DocstringCode {
     }
 }
 
+impl fmt::Display for DocstringCode {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Disabled => write!(f, "disabled"),
+            Self::Enabled => write!(f, "enabled"),
+        }
+    }
+}
+
 #[derive(Copy, Clone, Default, Eq, PartialEq, CacheKey)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[cfg_attr(feature = "serde", serde(rename_all = "lowercase"))]
@@ -323,8 +376,8 @@ pub enum DocstringCodeLineWidth {
     Dynamic,
 }
 
-impl std::fmt::Debug for DocstringCodeLineWidth {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+impl fmt::Debug for DocstringCodeLineWidth {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
             DocstringCodeLineWidth::Fixed(v) => v.value().fmt(f),
             DocstringCodeLineWidth::Dynamic => "dynamic".fmt(f),
@@ -332,6 +385,15 @@ impl std::fmt::Debug for DocstringCodeLineWidth {
     }
 }
 
+impl fmt::Display for DocstringCodeLineWidth {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Fixed(width) => width.fmt(f),
+            Self::Dynamic => write!(f, "dynamic"),
+        }
+    }
+}
+
 /// Responsible for deserializing the `DocstringCodeLineWidth::Dynamic`
 /// variant.
 fn deserialize_docstring_code_line_width_dynamic<'de, D>(d: D) -> Result<(), D::Error>
@@ -349,3 +411,22 @@ where
         )),
     }
 }
+
+#[derive(CacheKey, Clone, Copy, Debug, PartialOrd, Ord, PartialEq, Eq, Default)]
+#[cfg_attr(
+    feature = "serde",
+    derive(serde::Serialize, serde::Deserialize),
+    serde(rename_all = "lowercase")
+)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub enum PythonVersion {
+    Py37,
+    // Make sure to also change the default for `ruff_linter::settings::types::PythonVersion`
+    // when changing the default here.
+    #[default]
+    Py38,
+    Py39,
+    Py310,
+    Py311,
+    Py312,
+}
