@@ -700,7 +700,7 @@ impl<'source> Lexer<'source> {
                 }
                 Some('\r' | '\n') if !triple_quoted => {
                     if let Some(fstring) = self.fstrings.current() {
-                        // When we are in an f-string, check whether does the initial quote
+                        // When we are in an f-string, check whether the initial quote
                         // matches with f-strings quotes and if it is, then this must be a
                         // missing '}' token so raise the proper error.
                         if fstring.quote_char() == quote && !fstring.is_triple_quoted() {
@@ -708,7 +708,7 @@ impl<'source> Lexer<'source> {
                                 error: LexicalErrorType::FStringError(
                                     FStringErrorType::UnclosedLbrace,
                                 ),
-                                location: self.offset() - fstring.quote_size(),
+                                location: self.offset() - TextSize::new(1),
                             });
                         }
                     }
@@ -732,7 +732,7 @@ impl<'source> Lexer<'source> {
                 Some(_) => {}
                 None => {
                     if let Some(fstring) = self.fstrings.current() {
-                        // When we are in an f-string, check whether does the initial quote
+                        // When we are in an f-string, check whether the initial quote
                         // matches with f-strings quotes and if it is, then this must be a
                         // missing '}' token so raise the proper error.
                         if fstring.quote_char() == quote
@@ -742,7 +742,7 @@ impl<'source> Lexer<'source> {
                                 error: LexicalErrorType::FStringError(
                                     FStringErrorType::UnclosedLbrace,
                                 ),
-                                location: self.offset() - fstring.quote_size(),
+                                location: self.offset(),
                             });
                         }
                     }
@@ -1293,7 +1293,7 @@ impl FusedIterator for Lexer<'_> {}
 /// [lexer] implementation.
 ///
 /// [lexer]: crate::lexer
-#[derive(Debug, PartialEq)]
+#[derive(Debug, Clone, PartialEq)]
 pub struct LexicalError {
     /// The type of error that occurred.
     pub error: LexicalErrorType,
@@ -1308,8 +1308,33 @@ impl LexicalError {
     }
 }
 
+impl std::ops::Deref for LexicalError {
+    type Target = LexicalErrorType;
+
+    fn deref(&self) -> &Self::Target {
+        &self.error
+    }
+}
+
+impl std::error::Error for LexicalError {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        Some(&self.error)
+    }
+}
+
+impl std::fmt::Display for LexicalError {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        write!(
+            f,
+            "{} at byte offset {}",
+            &self.error,
+            u32::from(self.location)
+        )
+    }
+}
+
 /// Represents the different types of errors that can occur during lexing.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, Clone, PartialEq)]
 pub enum LexicalErrorType {
     // TODO: Can probably be removed, the places it is used seem to be able
     // to use the `UnicodeError` variant instead.
@@ -1350,6 +1375,8 @@ pub enum LexicalErrorType {
     OtherError(String),
 }
 
+impl std::error::Error for LexicalErrorType {}
+
 impl std::fmt::Display for LexicalErrorType {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
@@ -1449,7 +1476,7 @@ impl Radix {
             Radix::Binary => matches!(c, '0'..='1'),
             Radix::Octal => matches!(c, '0'..='7'),
             Radix::Decimal => c.is_ascii_digit(),
-            Radix::Hex => matches!(c, '0'..='9' | 'a'..='f' | 'A'..='F'),
+            Radix::Hex => c.is_ascii_hexdigit(),
         }
     }
 }
@@ -1471,9 +1498,12 @@ fn is_unicode_identifier_start(c: char) -> bool {
 // Checks if the character c is a valid continuation character as described
 // in https://docs.python.org/3/reference/lexical_analysis.html#identifiers
 fn is_identifier_continuation(c: char) -> bool {
-    match c {
-        'a'..='z' | 'A'..='Z' | '_' | '0'..='9' => true,
-        c => is_xid_continue(c),
+    // Arrange things such that ASCII codepoints never
+    // result in the slower `is_xid_continue` getting called.
+    if c.is_ascii() {
+        matches!(c, 'a'..='z' | 'A'..='Z' | '_' | '0'..='9')
+    } else {
+        is_xid_continue(c)
     }
 }
 
@@ -2168,13 +2198,17 @@ f"{(lambda x:{x})}"
         assert_debug_snapshot!(lex_jupyter_source(source));
     }
 
+    fn lex_error(source: &str) -> LexicalError {
+        match lex(source, Mode::Module).find_map(Result::err) {
+            Some(err) => err,
+            _ => panic!("Expected at least one error"),
+        }
+    }
+
     fn lex_fstring_error(source: &str) -> FStringErrorType {
-        match lex(source, Mode::Module).find_map(std::result::Result::err) {
-            Some(err) => match err.error {
-                LexicalErrorType::FStringError(error) => error,
-                _ => panic!("Expected FStringError: {err:?}"),
-            },
-            _ => panic!("Expected atleast one FStringError"),
+        match lex_error(source).error {
+            LexicalErrorType::FStringError(error) => error,
+            err => panic!("Expected FStringError: {err:?}"),
         }
     }
 
@@ -2219,4 +2253,25 @@ f"{(lambda x:{x})}"
             UnterminatedTripleQuotedString
         );
     }
+
+    #[test]
+    fn test_fstring_error_location() {
+        assert_debug_snapshot!(lex_error("f'{'"), @r###"
+        LexicalError {
+            error: FStringError(
+                UnclosedLbrace,
+            ),
+            location: 4,
+        }
+        "###);
+
+        assert_debug_snapshot!(lex_error("f'{'α"), @r###"
+        LexicalError {
+            error: FStringError(
+                UnclosedLbrace,
+            ),
+            location: 6,
+        }
+        "###);
+    }
 }
